******************************************************
*
* THERMO.PRG
*
* Copyright 1990, Tim Rossiter
*
******************************************************

parameters p1, p2, p3

* Program accepts at most three parameters:
*
*   - To initialize thermometer:
*       p1 = completed thermometer value
*
*   - To increment thermometer
*       p1 = string to print on first line (or null string)
*       p2 = string to print on second line (or null string)
*       p3 = current thermometer value
*
*   - To clear thermometer
*       pass no parameters
*

* store current value of TALK and shut it off
private stalk
if set( [TALK]) = [ON]
  set talk off
  stalk = [ON]
else
  stalk = [OFF]
endif

if type( [p1]) = [C] .and. wexist( [THERMO])
 
   * update thermometer

   private th_part, s1_same, s2_same, th_same, wout, woutvis

   * compute the portion of the thermometer to display
   if th_total = 0

     * we don't want to divide by zero
     th_part = th_max

   else

     * iif() makes sure mercury doesn't burst out the top
     th_part = round( ;
      ( iif( p3 > th_total, 1, p3 / th_total)) * th_max, 0)

   endif
   
   * test to see if anything has changed since the last time
   s1_same = ( p1 == th_lasts1)
   s2_same = ( p2 == th_lasts2)
   th_same = ( th_part == th_lastp)

   * check to see if nothing changed since last time
   if !( s1_same .and. s2_same .and. th_same)
 
     * save the current output window and its status
     wout = woutput()
     if wout # [THERMO] .and. !empty( wout)
       woutvis = wvisible( wout)
     endif

     * if the thermometer is not already on...
     if wout # [THERMO]

       if wvisible( [THERMO])

         * activate the thermometer
         activate window thermo

       else

         * activate the thermometer without displaying it
         activate window thermo noshow

       endif

     endif

     * following code test to see if things have changed
     *  since last update of thermometer -- since repeated
     *  calls to the thermometer do not always change the
     *  screen, this speeds up the thermometer by eliminating
     *  unnecessary screen i/o.

     * print the strings if different, pad on the right to
     *  clear any previous strings
     if !s1_same
       @ 0, 1 say left(padr( p1, th_width, [ ]), th_width-2)
       th_lasts1 = p1
     endif
     if !s2_same
       @ 1, 1 say left(padr( p2, th_width, [ ]), th_width-2)
       th_lasts2 = p2
     endif

     * print the mercury, if it has changed since last time
     if !th_same

       @ 3, 3 say padr( replicate( chr( 219),;
        int( th_part / 2)) + iif( mod( th_part, 2) = 1,;
        chr( 221), []), th_width - 6)

       th_lastp = th_part

     endif

     * if some other window was activate before...
     if wout # [THERMO]

       * if screen was previously active...
       if empty( wout)

         activate screen

       else

         * reactiave previous window
         if woutvis

           * previous window was visible
           activate window &wout

         else

           * previous window was hidden, activate it noshow
           activate window &wout noshow

         endif

       endif

     endif

   endif

else

  if type( [p1]) = [N]

    * intialize the thermomter

    private th_r, th_c

    * public variables need later to increment thermometer
    public th_total, th_width, th_height,;
      th_lastp, th_lasts1, th_lasts2, th_max

    * set size of thermometer
    th_width  = 56
    th_height =  5

    * store the thermometer total
    th_total  = p1

    * initialize 'last update' variables
    th_lastp  = 0
    th_lasts1 = []
    th_lasts2 = []

    * compute resolution of the thermometer
    th_max    = ( th_width - 6) * 2
    
    * upper right corner of thermometer window
    th_r = int( ( srows() - th_height) / 2) - 1
    th_c = int( ( scols() - th_width ) / 2) - 1

    * define the thermometer window
    define window thermo from th_r, th_c;
      to th_r + th_height + 1, th_c + th_width + 1;
      in screen double;
      noclose float nogrow noshadow nozoom;
      color scheme 5

    * draw the empty thermometer
    activate window thermo      
    @ 2, 1 to 4, th_width - 2
    
  else
      
    * zero parameters passed, close thermo down
    if wexist( [THERMO])
      release window thermo
    endif
    release th_total, th_width, th_height,;
      th_lastp, th_lasts1, th_lasts2, th_max

  endif

endif

* turn talk back on, if it was on to start
if stalk = [ON]
  set talk on
endif

* returns a value, so thermometer can be called from iif()
return .t.
